﻿-- FSwapper 3: Swaps between weapons and fishing pole depending on combat status
--
-- This mod depends on certain Ace-3 libraries, and LibDataBroker (all included)
-- Other than that, this mod is self-sufficient (no other depends)

local FSwapper3 = LibStub("AceAddon-3.0"):GetAddon("FSwapper3");
local L = LibStub("AceLocale-3.0"):GetLocale("FSwapper3", true);

local SkillBroker = FSwapper3.SkillBroker;
local FS3GUI = FSwapper3.FS3GUI;
local EquipList;

local SlashCmdList = SlashCmdList;
local _G = _G;

-- Global Functions Made Local
--[[
IsFishingLoot
GetSkillLineInfo
GetNumSkillLines
GetNumTrackingTypes
GetTrackingInfo

GetInventoryItemLink
GetItemInfo

UnitChannelInfo
UnitAffectingCombat

bit

PlaySoundFile

GetTime

GetNumLootItems
LootSlotIsCoin
LootSlot
GetLootSlotLink
]]

-- LASTERR reference
-- 0 No Error
-- 1 Weapons Not Set
-- 2 Pole Not Set
-- 3 Macro Modifier Invalid
-- 4 Auto Auto Loot (Player used Auto loot with FSwapper's Auto Loot)

-- Main functions

function FSwapper3:OnInitialize()
	FSwapper3.defaults = {
		char = {
			Enable = true,
			Notify = nil,
			bNotify = false,
			MainHand = nil,
			OffHand = nil,
			Pole = nil,
			FSTooltip = 0,
			Sound = true,
			Track = false,
			AutoLoot = {
				Enable = false,
				Quest = false,
				Contest = false,
				Quality0 = false,
				Quality1 = false,
				Quality2 = false,
				Quality3 = false,
				Quality4 = false,
				Quality5 = false,
				Quality6 = false,
				Quality7 = false,
				GoldCoin = false,
				SilverCoin = false,
				CopperCoin = false,
			},
			Equipment = {
				[1] = {
					Enable = false,
				},
				[2] = {
					Enable = false,
				},
				[3] = {
					Enable = false,
				},
				[4] = {
					Enable = false,
				},
				[5] = {
					Enable = false,
				},
			},
		},
	}
	FSwapper3.db = LibStub("AceDB-3.0"):New("FSwapper3DB", FSwapper3.defaults);
end

function FSwapper3:OnEnable()
	local _;
	local options = {
		name = "FSwapper3",
		handler = FSwapper3,
		type = 'group',
		args = {
			[GAMEOPTIONS_MENU] = {
				order = 1,
				type = "execute",
				name = GAMEOPTIONS_MENU,
				desc = GAMEOPTIONS_MENU,
				func = FS3GUI.Toggle,
			},
			
			GoFish = {
				order = 2,
				type = "input",
				name = "GoFish",
				desc = format(L["Equip your last used fishing pole. Use \"%s\" for macro help."], L["MacroHelp"]),
				set = 'EquipPole',
			},

			GoFight = {
				order = 3,
				type = "input",
				name = "GoFight",
				desc = format(L["Equip your last used weapons. Use \"%s\" for macro help."], L["MacroHelp"]),
				set = 'EquipWeapons',
			},

			DoSwap = {
				order = 4,
				type = "input",
				name = "DoSwap",
				desc = format(L["Swap between your last used fishing pole, and your last used weapons. Use \"%s\" for macro help."], L["MacroHelp"]),
				set = 'SwapSet',
			},

			[L["MacroHelp"]] = {
				order = 5,
				type = "execute",
				name = L["MacroHelp"],
				desc = L["Learn about useful modifiers for the macro commands."],
				func = 'MacroUsage',
			},
		},
	}
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FSwapper3", options, {"fswapper3","fswapper"});

	if (FSwapper3.db.char.Enable) then
		FSwapper3:Print(L["FSwapper3 is |cff00ff00enabled|r! Use \"/fswapperg\" to disable it."]);
	else
		FSwapper3:Print(L["FSwapper3 is |cffff0000disabled|r! Use \"/fswapperg\" to enable it."]);
	end
	
	if (FSwapper3.db.char.Notify) then
		if (FSwapper3.db.char.Notify == 2) then
			FSwapper3.db.char.bNotify = true;
		end
		FSwapper3.db.char.Notify = nil;
	end
	
	EquipList = FSwapper3.db.char.Equipment;
	EquipList[1].Slot = "HEADSLOT";
	EquipList[2].Slot = "FEETSLOT";
	EquipList[3].Slot = "HANDSSLOT";
	EquipList[4].Slot = "TRINKET0SLOT";
	EquipList[5].Slot = "TRINKET1SLOT";
	for i, v in ipairs(EquipList) do
		if (v.Slot == "TRINKET0SLOT") then
			v.Name = TRINKET0SLOT .. " 1";
		elseif (v.Slot == "TRINKET1SLOT") then
			v.Name = TRINKET1SLOT .. " 2";
		else
			v.Name = getglobal(v.Slot);
		end
	end

	FSwapper3.WasFishing = false;
	
	FS3GUI:DoInit();

	FSwapper3:SetWeapons();

	if ((FSwapper3.db.char.MainHand == nil) or (string.len(FSwapper3.db.char.MainHand) == 0)) then
		FSwapper3:Print(L["|cffff0000Warning:|r You entered the world without weapons. Equip weapons to set them."]);
	end

	FSwapper3:RegisterEvent("UNIT_INVENTORY_CHANGED");
	FSwapper3:RegisterEvent("PLAYER_REGEN_ENABLED");
	FSwapper3:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	FSwapper3:RegisterEvent("CHAT_MSG_SKILL");
	FSwapper3:RegisterEvent("SKILL_LINES_CHANGED");
	FSwapper3:RegisterEvent("LOOT_OPENED");
	
	SlashCmdList['FSwapper3_GUI'] = FS3GUI.Toggle;

	_G['SLASH_FSwapper3_GUI1'] = '/fswapperg';
	_G['SLASH_FSwapper3_GUI2'] = '/fs3g';
end

function FSwapper3:OnDisable()
    -- Called when the addon is disabled
end

function FSwapper3:SetEnable(info, value)
	if (FSwapper3.db.char.Enable) then
		FSwapper3:Print(L["FSwapper3 has been |cffff0000disabled|r for this character."]);
		FSwapper3.WasFishing = false;
		FSwapper3:PlaySound(FSwapper3.Sounds.Disable);
	else
		FSwapper3:Print(L["FSwapper3 has been |cff00ff00enabled|r for this character."]);
		FSwapper3:PlaySound(FSwapper3.Sounds.Enable);
	end
	FSwapper3.db.char.Enable = not FSwapper3.db.char.Enable

	FSwapper3:SetWeapons();
	FS3GUI.UpdateTab();
end

function FSwapper3:SetWeapons()
	local itemname, itemlink, itemsubtype, _;

	itemlink = GetInventoryItemLink("player", FSwapper3.nMAINHANDSLOT);
	if (itemlink) then
		itemname, itemlink, _, _, _, _, itemsubtype = GetItemInfo(itemlink);
		if (itemsubtype == FSwapper3.LFISHINGPOLES) then
			if (FSwapper3.db.char.Pole ~= itemlink) then
				FSwapper3.db.char.Pole = itemlink;
			end
			
			if (FSwapper3.WasFishing) then
				FSwapper3.WasFishing = false;
			end
			
			if ((FSwapper3.db.char.Track) and (FSwapper3.WasTracking == nil)) then
				FSwapper3.WasTracking = FSwapper3:GetTracking();
				FSwapper3:SetTracking(FSwapper3.LFINDFISH);
			end
		else
			if (FSwapper3.db.char.MainHand ~= itemlink) then
				FSwapper3.db.char.MainHand = itemlink;
			end

			itemlink = GetInventoryItemLink("player", FSwapper3.nSECONDARYHANDSLOT);
			if (itemlink) then
				itemname, itemlink = GetItemInfo(itemlink);
				if (FSwapper3.db.char.OffHand ~= itemlink) then
					FSwapper3.db.char.OffHand = itemlink;
				end
			else
				FSwapper3.db.char.OffHand = nil;
			end
			
			if ((FSwapper3.db.char.Track) and (FSwapper3.WasTracking ~= nil)) then
				FSwapper3:SetTracking(FSwapper3.WasTracking);
				FSwapper3.WasTracking = nil;
			end
		end
	end
		
	FSwapper3:DoDBUpdate();
	FS3GUI.UpdateTab(1);
end

function FSwapper3:UNIT_INVENTORY_CHANGED(event, unit)
	if (unit ~= "player") then
		return;
	end

	FSwapper3:SetWeapons();
end

function FSwapper3:PLAYER_REGEN_ENABLED()
	if (not FSwapper3.db.char.Enable) then
		return;
	end

	if (FSwapper3.WasFishing) then
		if (FSwapper3.db.char.bNotify) then
			FSwapper3:Print(L["|cff0000ffLeaving combat!|r Swapping to fishing pole."]);
		end
		FSwapper3:EquipPole();
	end
end

function FSwapper3:COMBAT_LOG_EVENT_UNFILTERED(event, _, _, _, _, sourceFlags, _, _, destFlags)
	local itemname, itemlink, fishingpolename;
	if (not FSwapper3.db.char.Enable) then
		return;
	end

	if (not UnitAffectingCombat("player")) then
		return;
	end

	-- If we're channeling (still fishing)
	if (UnitChannelInfo("player")) then
		return;
	end

	if ((FSwapper3.db.char.Pole == nil) or (string.len(FSwapper3.db.char.Pole) == 0)) then
		return; -- obviously wasn't fishing if we don't know the fishing pole
	end
	fishingpolename = GetItemInfo(FSwapper3.db.char.Pole);

	itemlink = GetInventoryItemLink("player", FSwapper3.nMAINHANDSLOT);
	if (itemlink) then
		itemname = GetItemInfo(itemlink);
	end

	if (itemname ~= fishingpolename) then
		return; -- already swapped, or wasn't fishing
	end

	-- Make sure YOU did something to an ENEMY, or an ENEMY did something to YOU
	-- This prevents swapping if a nearby party member forces you into combat by them attacking something.
	if (((bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) == 0) or
	     (bit.band(destFlags, COMBATLOG_OBJECT_REACTION_FRIENDLY) ~= 0)) and
	    ((bit.band(sourceFlags, COMBATLOG_OBJECT_REACTION_FRIENDLY) ~= 0) or
	     (bit.band(destFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) == 0)))
	then
		return;
	end
    
    FSwapper3:EquipWeapons(nil, nil, true);
end

function FSwapper3:CHAT_MSG_SKILL()
	FSwapper3:DoDBUpdate();
end

function FSwapper3:SKILL_LINES_CHANGED()
	FSwapper3:DoDBUpdate();
end

function FSwapper3:LOOT_OPENED(info, auto)
	local AutoLoot = FSwapper3.db.char.AutoLoot;
	if ((not AutoLoot.Enable) or (not IsFishingLoot())) then
		return;
	end
	if ((auto ~= 0) and AutoLoot.Enable) then
		FSwapper3:SetError(4, 5, L["|cffff0000Warning:|r Disable WoW's Auto Loot to use FSwapper3's Auto Loot"]);
		return;
	end
	
	for i = 1, GetNumLootItems() do
		if (LootSlotIsCoin(i) or FSwapper3:CheckLoot(i, AutoLoot)) then
			LootSlot(i);
		end
	end	
end

function FSwapper3:CheckLoot(index, AutoLoot)
	local itemlink, itemid, itemquality, itemtype, _;
	itemlink = GetLootSlotLink(index);
	if (not itemlink) then
		return false;
	end
	itemid = tonumber((select(2,strsplit(":", itemlink))));
	_, _, itemquality, _, _, itemtype = GetItemInfo(itemlink);
	
	--quest, contest, quality, gold, silver, copper
	if ((AutoLoot.Quest) and (itemtype == FSwapper3.LQUEST)) then
		return true;
	end
	
	if (AutoLoot.Contest) then
		-- 19803..19807 = Brownell's, INVALID, Keefer's, Dezian, Tastyfish
		if ((itemid >= 19803) and (itemid <= 19807)) then
			return true;
		end
	end
	
	if (AutoLoot["Quality" .. itemquality]) then
		return true;
	end
	
	if (AutoLoot.GoldCoin) then
		if ((itemid >= 43627) and (itemid <= 43641)) then
			return true;
		end
	end
	
	if (AutoLoot.SilverCoin) then
		if ((itemid >= 43643) and (itemid <= 43644)) then
			return true;
		end
		if ((itemid >= 43675) and (itemid <= 43687)) then
			return true;
		end
	end
	
	if (AutoLoot.CopperCoin) then
		if ((itemid >= 43701) and (itemid <= 43724)) then
			return true;
		end
	end
	
	return false;
end

function FSwapper3:UpdateSkill()
	local index, tempindex, tempskill, tempbonus, tempmax, _;
	if ((SkillBroker.SkillIndex == -1) or (GetSkillLineInfo(SkillBroker.SkillIndex) ~= FSwapper3.LFISHINGSPELL)) then
		tempindex = -1;
		for index = 1, GetNumSkillLines() do
			if (GetSkillLineInfo(index) == FSwapper3.LFISHINGSPELL) then
				tempindex = index;
				break;
			end
		end
	end
   
    -- If we don't have the fishing skill, and it's news to us
    if ((tempindex == -1) and (tempindex ~= SkillBroker.SkillIndex)) then
        SkillBroker.SkillIndex = -1;
		SkillBroker.SkillStr = FSwapper3.FISHINGSKILL_NULL;
		return;
	end
    
    if (tempindex ~= nil) then
        SkillBroker.SkillIndex = tempindex;
    end

	_, _, _, tempskill, _, tempbonus, tempmax = GetSkillLineInfo(SkillBroker.SkillIndex);

    -- Check if anything changed (do we really need to reformat strings?)
    if ((tempskill ~= SkillBroker.Skill) or (tempbonus ~= SkillBroker.Bonus) or (tempmax ~= SkillBroker.SkillMax)) then
        SkillBroker.Skill = tempskill;
        SkillBroker.Bonus = tempbonus;
        SkillBroker.SkillMax = tempmax;
	    if (SkillBroker.Bonus > 0) then
	    	SkillBroker.SkillStr = format(FSwapper3.FISHINGSKILL_SKILL_BONUS_MAX, SkillBroker.Skill, SkillBroker.Bonus, SkillBroker.SkillMax);
    	else
		    SkillBroker.SkillStr = format(FSwapper3.FISHINGSKILL_SKILL_MAX, SkillBroker.Skill, SkillBroker.SkillMax);
	    end
    end
end

function FSwapper3:GetTracking()
	local name, active, _;
	for i = 1, GetNumTrackingTypes() do
		name, _, active = GetTrackingInfo(i);
		if (active) then
			return name;
		end
	end
	return nil;
end

function FSwapper3:SetTracking(track)
	local name;
	for i = 1, GetNumTrackingTypes() do
		name = GetTrackingInfo(i);
		if (name == track) then
			SetTracking(i);
			break;
		end
	end
end

function FSwapper3:EquipWeapons(info, modifiers, wasauto)
	local itemlink, itemsubtype, _;

	if (not FSwapper3:CheckModifiers(modifiers)) then
		return;
	end

	if ((FSwapper3.db.char.MainHand == nil) or (string.len(FSwapper3.db.char.MainHand) == 0)) then
		FSwapper3:SetError(1, 5, L["|cffff0000Error:|r Main Hand is |cffff0000[Not Set]|r! Equip weapons manually to set it."]);
		return;
	end

	itemlink = GetInventoryItemLink("player", FSwapper3.nMAINHANDSLOT);
	if (itemlink) then
		_, _, _, _, _, _, itemsubtype = GetItemInfo(itemlink);
	end
    
    if (wasauto) then
        if ((not FSwapper3.WasFishing) and (FSwapper3.db.char.bNotify)) then -- only say combat detected once
		    FSwapper3:Print(L["|cffff0000Combat detected!|r Swapping to weapons."]);
	    end
        FSwapper3.WasFishing = true;
    else
        FSwapper3.WasFishing = false;
    end

	FSwapper3:DoDBUpdate();

    FSwapper3.LASTERR = 0;
	if (itemsubtype == FSwapper3.LFISHINGPOLES) then
		FSwapper3:PlaySound(FSwapper3.Sounds.Weapons);
		EquipItemByName(FSwapper3.db.char.MainHand, FSwapper3.nMAINHANDSLOT);
		if ((FSwapper3.db.char.OffHand ~= nil) and (string.len(FSwapper3.db.char.OffHand) > 0)) then
			EquipItemByName(FSwapper3.db.char.OffHand, FSwapper3.nSECONDARYHANDSLOT);
		end
	end
	
	if (not wasauto) then
		for i, v in ipairs(EquipList) do
			itemlink = GetInventoryItemLink("player", FSwapper3["n" .. v.Slot]);
			if (v.Normal and (itemlink == v.Fishing)) then
				EquipItemByName(v.Normal, FSwapper3["n" .. v.Slot]);
			end
		end
	end
end

function FSwapper3:EquipPole(info, modifiers)
	local itemlink, itemsubtype, _;

	if (not FSwapper3:CheckModifiers(modifiers)) then
		return;
	end

	if ((FSwapper3.db.char.Pole == nil) or (string.len(FSwapper3.db.char.Pole) == 0)) then
		FSwapper3:SetError(2, 5, L["|cffff0000Error:|r Fishing Pole is |cffff0000[Not Set]|r! Equip one manually to set it."]);
        return;
	end

	itemlink = GetInventoryItemLink("player", FSwapper3.nMAINHANDSLOT);
	if (itemlink) then
		_, _, _, _, _, _, itemsubtype = GetItemInfo(itemlink);
	end

	FSwapper3:DoDBUpdate();

    FSwapper3.LASTERR = 0;
	if (itemsubtype ~= FSwapper3.LFISHINGPOLES) then
		FSwapper3:PlaySound(FSwapper3.Sounds.Fishing);
		EquipItemByName(FSwapper3.db.char.Pole, FSwapper3.nMAINHANDSLOT);
	end
	
	if (not FSwapper3.WasFishing) then
		for i, v in ipairs(EquipList) do
			if (v.Enable and v.Fishing) then
				itemlink = GetInventoryItemLink("player", FSwapper3["n" .. v.Slot]);
				if (itemlink ~= v.Fishing) then
					v.Normal = itemlink;
					EquipItemByName(v.Fishing, FSwapper3["n" .. v.Slot]);
				end
			else
				v.Normal = nil;
			end
		end
	end
end

function FSwapper3:SwapSet(info, modifiers)
	local itemlink, itemsubtype, _;

	if (not FSwapper3:CheckModifiers(modifiers)) then
		return;
	end

	itemlink = GetInventoryItemLink("player", FSwapper3.nMAINHANDSLOT);
	if (itemlink) then
		_, _, _, _, _, _, itemsubtype = GetItemInfo(itemlink);
	end

	FSwapper3:DoDBUpdate();

	if (itemsubtype == FSwapper3.LFISHINGPOLES) then
		FSwapper3:EquipWeapons();
	else
		FSwapper3:EquipPole();
	end
end

function FSwapper3:MacroUsage()
	FSwapper3:Print(L["Macro Usage:"]);
	FSwapper3:Print(format(L["Acceptable modifiers are: %s"], "Shift, Ctrl, Alt, LeftButton, RightButton, MiddleButton, Button4, Button5"));
	FSwapper3:Print(format(L["You can put \"%s\" in front of a modifier for its inverse. For example \"%s\" would equip weapons if the left button was not clicked."], "no", "/FSwapper3 gofight noLeftButton"));
	FSwapper3:Print(L["See FSwapper3's Curse page for more advanced macro info."]);
end

function FSwapper3:CheckModifiers(modifiers)
	local isshift, isctrl, isalt, isbutton = IsShiftKeyDown(), IsControlKeyDown(), IsAltKeyDown(), GetMouseButtonClicked();
	local doshift, doctrl, doalt, dobutton = 0, 0, 0, 0;
	local pos = 0;
    
    if (not modifiers) then
        return true;
    end
    
	while(true) do
		local word, mod;
		local neg = false;
		_,pos,word = string.find(modifiers, "^ *([^%s]+) *", pos+1);
		if(not word) then
			break;
		end
		if (string.len(word) > 2) then
			neg = (string.lower(string.sub(word, 1, 2)) == "no");
		end
		if (neg) then
			mod = string.lower(string.sub(word, 3));
		else
			mod = string.lower(word);
		end
		if (mod == "shift") then
			if (neg) then
				doshift = -1;
			else
				doshift = 1;
			end
		elseif (mod == "ctrl") then
			if (neg) then
				doctrl = -1;
			else
				doctrl = 1;
			end
		elseif (mod == "alt") then
			if (neg) then
				doalt = -1;
			else
				doalt = 1;
			end
		elseif ((mod == "btn") or (mod == "button")) then
			if (neg) then
				dobutton = -6;
			else
				dobutton = 6;
			end
		elseif ((mod == "btn1") or (mod == "button1") or (mod == "leftbtn") or (mod == "leftbutton")) then
			if (neg) then
				dobutton = -1;
			else
				dobutton = 1;
			end
		elseif ((mod == "btn2") or (mod == "button2") or (mod == "rightbtn") or (mod == "rightbutton")) then
			if (neg) then
				dobutton = -2;
			else
				dobutton = 2;
			end
		elseif ((mod == "btn3") or (mod == "button3") or (mod == "middlebtn") or (mod == "middlebutton")) then
			if (neg) then
				dobutton = -3;
			else
				dobutton = 3;
			end
		elseif ((mod == "btn4") or (mod == "button4")) then
			if (neg) then
				dobutton = -4;
			else
				dobutton = 4;
			end
		elseif ((mod == "btn5") or (mod == "button5")) then
			if (neg) then
				dobutton = -5;
			else
				dobutton = 5;
			end
		else
            if (FSwapper3:SetError(3, 5, format(L["|cffff0000Error:|r Macro modifier \"%s\" was invalid!"], word))) then
                FSwapper3:MacroUsage();
            end
			return false;
		end
	end

	if ((doshift == 1) and (not isshift)) then
		return false;
	elseif ((doshift == -1) and (isshift)) then
		return false;
	elseif ((doctrl == 1) and (not isctrl)) then
		return false;
	elseif ((doctrl == -1) and (isctrl)) then
		return false;
	elseif ((doalt == 1) and (not isalt)) then
		return false;
	elseif ((doalt == -1) and (isalt)) then
		return false;
	elseif ((dobutton == 1) and (isbutton ~= "LeftButton")) then
		return false;
	elseif ((dobutton == -1) and (isbutton == "LeftButton")) then
		return false;
	elseif ((dobutton == 2) and (isbutton ~= "RightButton")) then
		return false;
	elseif ((dobutton == -2) and (isbutton == "RightButton")) then
		return false;
	elseif ((dobutton == 3) and (isbutton ~= "MiddleButton")) then
		return false;
	elseif ((dobutton == -3) and (isbutton == "MiddleButton")) then
		return false;
	elseif ((dobutton == 4) and (isbutton ~= "Button4")) then
		return false;
	elseif ((dobutton == -4) and (isbutton == "Button4")) then
		return false;
	elseif ((dobutton == 5) and (isbutton ~= "Button5")) then
		return false;
	elseif ((dobutton == -5) and (isbutton == "Button5")) then
		return false;
	elseif ((dobutton == 6) and (isbutton == nil)) then
		return false;
	elseif ((dobutton == -6) and (isbutton ~= nil)) then
		return false;
	else
		return true;
	end
end

function FSwapper3:PlaySound(sound)
	if (sound == nil) then
		return;
	end
	if (FSwapper3.db.char.Sound) then
		PlaySoundFile(sound);
	end
end

function FSwapper3:SetError(err, elap, str)
    if ((FSwapper3.LASTERR ~= err) or ((GetTime() - FSwapper3.LASTERRT) > elap)) then
        FSwapper3.LASTERR = err;
        FSwapper3.LASTERRT = GetTime();
        if (str) then
            FSwapper3:Print(str);
            if (FSwapper3.db.char.Sound) then
                FSwapper3:PlaySound(FSwapper3.Sounds.Error);
            end
        end
        return true;
    end
    return false;
end