﻿local FSwapper3 = LibStub("AceAddon-3.0"):GetAddon("FSwapper3");
local L = LibStub("AceLocale-3.0"):GetLocale("FSwapper3", true);

local FS3GUI = FSwapper3.FS3GUI;
local EquipList;

local TabFrame = FS3GUI.TabFrame;
local WeaponFrame = FS3GUI.SubFrame[1];
local OptionsFrame = FS3GUI.SubFrame[2];
local AutoLootFrame = FS3GUI.SubFrame[3];
local EquipmentFrame = FS3GUI.SubFrame[4];

function EquipmentFrame:DoInit()
	EquipmentFrame.DidInit = true;
	
	EquipmentFrame:SetFrameStrata("DIALOG");
	EquipmentFrame:SetWidth(FS3GUI.Width);
	EquipmentFrame:SetBackdrop({
			bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
			edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
			tile = true, tileSize = 32, edgeSize = 32, 
			insets = { left = 8, right = 8, top = 8, bottom = 8 }
		});
	EquipmentFrame:SetBackdropColor(0,0,0,1);
	EquipmentFrame:EnableMouse();
	EquipmentFrame:SetPoint("TOPLEFT", TabFrame, "BOTTOMLEFT", 0, 12);
	
	EquipmentFrame.SlotList = {};
	EquipmentFrame.SlotList.ypos = 13;
	
	EquipList = FSwapper3.db.char.Equipment;
	for i, v in ipairs(EquipList) do
		EquipmentFrame:AddSlot(EquipmentFrame.SlotList);
	end
	
	EquipmentFrame:SetHeight(EquipmentFrame.SlotList.ypos + 13);
end

function EquipmentFrame:AddSlot(list)
	local index = (#list) + 1;
	local maxwidth = FS3GUI.Width - 26;
	
	local SlotLine = {};
	local data = EquipList[index];
	SlotLine.BtnLink = nil;
	
	SlotLine.Label = EquipmentFrame:CreateFontString("$parent_SlotLabel" .. index,"ARTWORK","GameFontNormal");
	SlotLine.Label:SetText(data.Name);
	SlotLine.Label:SetPoint("TOPLEFT", EquipmentFrame, "TOPLEFT", 13, -(list.ypos));
	SlotLine.Label:SetWidth(maxwidth);
	SlotLine.Label:SetJustifyH("LEFT");
	
	list.ypos = list.ypos + SlotLine.Label:GetHeight();
	
	SlotLine.Tick = CreateFrame("CheckButton", nil, EquipmentFrame, "UICheckButtonTemplate");
	SlotLine.Tick:SetPoint("TOPLEFT", EquipmentFrame, "TOPLEFT", 13, -(list.ypos));
	SlotLine.Tick:SetScript("OnClick", function(self, button)
			if (button == "LeftButton") then
				EquipList[index].Enable = not EquipList[index].Enable;
				EquipmentFrame:DoUpdate();
			end
		end);
		
	SlotLine.LinkText = EquipmentFrame:CreateFontString("$parent_SlotLink" .. index,"ARTWORK","GameFontNormal");
	SlotLine.LinkText:SetPoint("TOPLEFT", SlotLine.Tick, "TOPRIGHT");
	SlotLine.LinkText:SetJustifyH("LEFT");
	SlotLine.LinkText:SetJustifyV("CENTER");
	SlotLine.LinkText:SetHeight(SlotLine.Tick:GetHeight());
	SlotLine.LinkText:SetWidth(maxwidth - SlotLine.Tick:GetWidth());
	
	SlotLine.LinkTT = CreateFrame("Frame", nil, EquipmentFrame);
	SlotLine.LinkTT:SetPoint("TOPLEFT", SlotLine.LinkText);
	SlotLine.LinkTT:EnableMouse();
	SlotLine.LinkTT:SetScript("OnEnter", function()
			GameTooltip:SetOwner(SlotLine.LinkTT, "ANCHOR_NONE");
			GameTooltip:SetPoint("TOPLEFT", EquipmentFrame, "TOPRIGHT");
			GameTooltip:ClearLines();
			if (EquipList[index].Fishing) then
				GameTooltip:SetHyperlink(EquipList[index].Fishing);
			else
				GameTooltip:AddLine(L["|cffff0000[Not Set]|r"]);
			end
			GameTooltip:Show();
		end);
	SlotLine.LinkTT:SetScript("OnLeave", function()
			GameTooltip:Hide();
		end);
		
	list.ypos = list.ypos + SlotLine.Tick:GetHeight();
	
	SlotLine.BagBtn = CreateFrame("Button", nil, EquipmentFrame, "UIPanelButtonTemplate");
	SlotLine.BagBtn:SetPoint("TOPLEFT", EquipmentFrame, "TOPLEFT", 13, -(list.ypos));
	SlotLine.BagBtn:SetText(EMPTY);
	SlotLine.BagBtn:SetHeight(SlotLine.BagBtn:GetTextHeight() + 10);
	SlotLine.BagBtn:SetWidth(SlotLine.BagBtn:GetTextWidth() + 20);
	SlotLine.BagBtn:SetScript("OnEnter", function()
			GameTooltip:SetOwner(SlotLine.BagBtn, "ANCHOR_NONE");
			GameTooltip:SetPoint("TOPLEFT", EquipmentFrame, "TOPRIGHT");
			GameTooltip:ClearLines();
			if (SlotLine.BtnLink) then
				GameTooltip:SetHyperlink(SlotLine.BtnLink);
			else
				GameTooltip:AddLine(EMPTY);
			end
			GameTooltip:Show();
		end);
	SlotLine.BagBtn:SetScript("OnLeave", function()
			GameTooltip:Hide();
		end);
	SlotLine.BagBtn:SetScript("OnClick", function(self, button)
			if (button == "LeftButton") then
				if (SlotLine.BtnLink) then
					data.Fishing = SlotLine.BtnLink;
					EquipmentFrame:DoUpdate();
				end
			end
		end);
	
	list.ypos = list.ypos + SlotLine.BagBtn:GetHeight();
	
	list[index] = SlotLine;
end

function EquipmentFrame:DoUpdate()
	for i, v in ipairs(EquipmentFrame.SlotList) do
		v.BtnLink = GetInventoryItemLink("player", FSwapper3["n" .. EquipList[i].Slot]);
		if (v.BtnLink) then
			v.BagBtn:SetText(v.BtnLink);
			v.BagBtn:SetWidth(v.BagBtn:GetTextWidth() + 20);
		else
			v.BagBtn:SetText(EMPTY);
			v.BagBtn:SetWidth(v.BagBtn:GetTextWidth() + 20);
		end
		
		v.Tick:SetChecked(EquipList[i].Enable);
		v.LinkText:SetText(EquipList[i].Fishing or L["|cffff0000[Not Set]|r"]);
		v.LinkTT:SetHeight(v.LinkText:GetHeight());
		v.LinkTT:SetWidth(v.LinkText:GetStringWidth());
	end
end