﻿local FSwapper3 = LibStub("AceAddon-3.0"):GetAddon("FSwapper3");
local L = LibStub("AceLocale-3.0"):GetLocale("FSwapper3", true);

FSwapper3.FS3GUI = {};
local FS3GUI = FSwapper3.FS3GUI;

local TabFrame = CreateFrame("Frame", nil, UIParent);
local WeaponFrame = CreateFrame("Frame", nil, TabFrame);
local OptionsFrame = CreateFrame("Frame", nil, TabFrame);
local AutoLootFrame = CreateFrame("Frame", nil, TabFrame);
local EquipmentFrame = CreateFrame("Frame", nil, TabFrame);

FS3GUI.TabFrame = TabFrame;
FS3GUI.SubFrame = {};
FS3GUI.SubFrame[1] = WeaponFrame;
FS3GUI.SubFrame[2] = OptionsFrame;
FS3GUI.SubFrame[3] = AutoLootFrame;
FS3GUI.SubFrame[4] = EquipmentFrame;

function FS3GUI:DoInit()		
	WeaponFrame.hTitle = ENCHSLOT_WEAPON;
	OptionsFrame.hTitle = GAMEOPTIONS_MENU;
	AutoLootFrame.hTitle = AUTO_LOOT_DEFAULT_TEXT;
	EquipmentFrame.hTitle = EQUIPMENT_MANAGER;
	
	WeaponFrame.DidInit = nil;
	OptionsFrame.DidInit = nil;
	AutoLootFrame.DidInit = nil;
	EquipmentFrame.DidInit = nil;
	
	TabFrame:DoInit();
end

function TabFrame:DoInit()
	local maxwidth, frameheight = 350, 13;
	FS3GUI.Width = maxwidth;
	FS3GUI.OpenTab = 1;
	
	TabFrame:SetFrameStrata("DIALOG");
	TabFrame:SetWidth(maxwidth);
	TabFrame:SetAttribute("UIPanelLayout-defined", true);
	TabFrame:SetAttribute("UIPanelLayout-enabled", true);
	TabFrame:SetAttribute("UIPanelLayout-area", "left");
	TabFrame:SetAttribute("UIPanelLayout-pushable", 5);
	TabFrame:SetAttribute("UIPanelLayout-width", maxwidth);
	TabFrame:SetAttribute("UIPanelLayout-whileDead", true);
	TabFrame:SetBackdrop({
			bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
			edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
			tile = true, tileSize = 32, edgeSize = 32, 
			insets = { left = 8, right = 8, top = 8, bottom = 8 }
		});
	TabFrame:SetBackdropColor(0,0,0,1);
	TabFrame:EnableMouse();
	TabFrame:SetScript("OnShow", function()
			FS3GUI:ShowTab();
		end);
	HideUIPanel(TabFrame);
	
	TabFrame.iddTitle = TabFrame:CreateFontString("$parent_Title","ARTWORK","GameFontNormal");
	TabFrame.iddTitle:SetText("FSwapper3 v" .. FSwapper3.FS3VERSION);
	TabFrame.iddTitle:SetPoint("TOPLEFT",TabFrame,"TOPLEFT",13,-frameheight);
	TabFrame.iddTitle:SetWidth(maxwidth - 26);
	TabFrame.iddTitle:SetJustifyH("CENTER");
	
	frameheight = frameheight + TabFrame.iddTitle:GetHeight() + 5;
	
	TabFrame.iddEnableBtn = CreateFrame("Button", nil, TabFrame, "UIPanelButtonTemplate");
	TabFrame.iddEnableBtn:SetText(ENABLE);
	TabFrame.iddEnableBtn:SetPoint("TOPRIGHT",TabFrame,"TOPRIGHT",-13,-frameheight);
	local EnableBtnWidth = TabFrame.iddEnableBtn:GetTextWidth() + 20;
	local BtnHeight = TabFrame.iddEnableBtn:GetTextHeight() + 10;
	TabFrame.iddEnableBtn:SetWidth(EnableBtnWidth);
	TabFrame.iddEnableBtn:SetHeight(BtnHeight);
	TabFrame.iddEnableBtn:SetScript("OnClick", function (self, button)
			if (button == "LeftButton") then
				FSwapper3:SetEnable();
				FS3GUI:UpdateTab();
			end
		end)
	
	TabFrame.iddCloseBtn = CreateFrame("Button", nil, TabFrame, "UIPanelCloseButton");
	TabFrame.iddCloseBtn:SetPoint("TOPRIGHT", TabFrame, "TOPRIGHT", -4, -4);
	
	TabFrame.iddTabBtn = {};
	local TabBtnx, TabBtncx, TabBtncy, TabWidth = 13, 0, BtnHeight, (maxwidth - 13 - EnableBtnWidth);
	for i, frame in ipairs(FS3GUI.SubFrame) do
		frame.DidInit = nil;
		TabFrame.iddTabBtn[i] = CreateFrame("Button", nil, TabFrame, "UIPanelButtonTemplate");
		local Btn = TabFrame.iddTabBtn[i];
		Btn:SetText(frame.hTitle);
		TabBtncx = Btn:GetTextWidth() + 20;
		Btn:SetWidth(TabBtncx);
		BtnHeight = Btn:GetTextHeight() + 10;
		Btn:SetHeight(BtnHeight);
		if ((TabBtnx + TabBtncx + 5) > TabWidth) then
			frameheight = frameheight + TabBtncy + 5;
			TabBtnx = 13;
			TabBtncy = BtnHeight;
		end
		Btn:SetPoint("TOPLEFT", TabFrame, "TOPLEFT", TabBtnx, -frameheight);
		TabBtnx = TabBtnx + TabBtncx + 5;
		if (BtnHeight > TabBtncy) then
			TabBtncy = BtnHeight;
		end
		Btn:SetScript("OnClick", function(self, button)
				if (button == "LeftButton") then
					FS3GUI:ShowTab(i);
				end
			end);
	end
	
	frameheight = frameheight + TabBtncy + 13;
	
	TabFrame:SetHeight(frameheight);
end

function FS3GUI:Toggle()
	if (TabFrame:IsShown()) then
		HideUIPanel(TabFrame);
	else
		ShowUIPanel(TabFrame);
	end
end

function FS3GUI:ShowTab(tab)
	if (not tab) then
		tab = FS3GUI.OpenTab;
	end
	if (not FS3GUI.SubFrame[tab].DidInit) then
		FS3GUI.SubFrame[tab].DoInit();
	end
	
	FS3GUI:UpdateTab(tab);
	
	for i, frame in ipairs(FS3GUI.SubFrame) do
		if (i == tab) then
			TabFrame.iddTabBtn[i]:SetButtonState("PUSHED", true);
			FS3GUI.OpenTab = tab;
			frame:Show();
		else
			TabFrame.iddTabBtn[i]:SetButtonState("NORMAL");
			frame:Hide();
		end
	end
end

function FS3GUI:UpdateTab(tab)
	if (not tab) then
		tab = FS3GUI.OpenTab;
	end
	if (not FS3GUI.SubFrame[tab].DidInit) then
		FS3GUI.SubFrame[tab].DoInit();
	end
	
	if (FSwapper3.db.char.Enable) then
		TabFrame.iddEnableBtn:SetButtonState("PUSHED", true);
	else
		TabFrame.iddEnableBtn:SetButtonState("NORMAL");
	end
	
	if (FS3GUI.SubFrame[tab].DoUpdate) then
		FS3GUI.SubFrame[tab].DoUpdate();
	end
end