﻿local FSwapper3 = LibStub("AceAddon-3.0"):GetAddon("FSwapper3");
local L = LibStub("AceLocale-3.0"):GetLocale("FSwapper3", true);

local FS3GUI = FSwapper3.FS3GUI;

local TabFrame = FS3GUI.TabFrame;
local WeaponFrame = FS3GUI.SubFrame[1];
local OptionsFrame = FS3GUI.SubFrame[2];
local AutoLootFrame = FS3GUI.SubFrame[3];
local EquipmentFrame = FS3GUI.SubFrame[4];

function WeaponFrame:DoInit()
	WeaponFrame.DidInit = true;
	
	WeaponFrame:SetFrameStrata("DIALOG");
	WeaponFrame:SetWidth(FS3GUI.Width);
	WeaponFrame:SetBackdrop({
			bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
			edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
			tile = true, tileSize = 32, edgeSize = 32, 
			insets = { left = 8, right = 8, top = 8, bottom = 8 }
		});
	WeaponFrame:SetBackdropColor(0,0,0,1);
	WeaponFrame:EnableMouse();
	WeaponFrame:SetPoint("TOPLEFT", TabFrame, "BOTTOMLEFT", 0, 12);
	
	WeaponFrame.WeaponList = {};
	WeaponFrame:AddWeaponLine(WeaponFrame.WeaponList, MAINHANDSLOT .. ": ", "MainHand", -13);
	WeaponFrame:AddWeaponLine(WeaponFrame.WeaponList, SECONDARYHANDSLOT .. ": ", "OffHand");
	WeaponFrame:AddWeaponLine(WeaponFrame.WeaponList, FSwapper3.LFISHINGPOLES .. ": ", "Pole");
	
end

function WeaponFrame:AddWeaponLine(list, label, value, ypos)
	local index = (#list) + 1;
	
	local WpnLine = {};
	WpnLine.Value = value;
	
	WpnLine.Label = WeaponFrame:CreateFontString("$parent_WpnLabel" .. index, "ARTWORK", "GameFontNormal");
	WpnLine.Text = WeaponFrame:CreateFontString("$parent_WpnText" .. index, "ARTWORK", "GameFontNormal");
	WpnLine.Tip = CreateFrame("Frame", nil, WeaponFrame);
	
	if (ypos) then
		WpnLine.Label:SetPoint("TOPLEFT", WeaponFrame, "TOPLEFT", 13, ypos);
	else
		WpnLine.Label:SetPoint("TOPLEFT", list[index-1].Label, "BOTTOMLEFT", 0, -5);
	end
	WpnLine.Label:SetText(label);
	WpnLine.Label:SetJustifyH("LEFT");
	WpnLine.Label:SetJustifyV("MIDDLE");
		
	WpnLine.Text:SetPoint("TOPLEFT", WpnLine.Label, "TOPRIGHT", 0, 0);
	WpnLine.Text:SetWidth(FS3GUI.Width - 26 - WpnLine.Label:GetWidth());
	WpnLine.Text:SetJustifyH("LEFT");
	
	WpnLine.Tip:SetPoint("TOPLEFT", WpnLine.Text);
	WpnLine.Tip:EnableMouse();
	
	WpnLine.Tip:SetScript("OnEnter", function()
			GameTooltip:SetOwner(WpnLine.Tip, "ANCHOR_NONE");
			GameTooltip:SetPoint("TOPLEFT", WpnLine.Tip, "BOTTOMLEFT");
			GameTooltip:ClearLines();
			if (FSwapper3.db.char[value]) then
				GameTooltip:SetHyperlink(FSwapper3.db.char[value]);
			else
				GameTooltip:AddLine(L["|cffff0000[Not Set]|r"]);
			end
			GameTooltip:Show();
		end);
	WpnLine.Tip:SetScript("OnLeave", function()
			GameTooltip:Hide();
		end);
		
	list[index] = WpnLine;
end

function WeaponFrame:DoUpdate()
	local frameheight, labelheight, textwidth, textheight, maxwidth = 13, 0, 0, 0, 0;
	for i, WpnLine in ipairs(WeaponFrame.WeaponList) do
		labelheight = WpnLine.Label:GetStringHeight();
		
		WpnLine.Text:SetText(FSwapper3.db.char[WpnLine.Value] or L["|cffff0000[Not Set]|r"]);
		textheight = WpnLine.Text:GetHeight();
		if (labelheight > textheight) then
			textheight = labelheight;
		end
		WpnLine.Label:SetHeight(textheight);
		
		maxwidth = WpnLine.Text:GetWidth();
		textwidth = WpnLine.Text:GetStringWidth();
		if (maxwidth < textwidth) then
			textwidth = maxwidth;
		end
		
		WpnLine.Tip:SetWidth(textwidth);
		WpnLine.Tip:SetHeight(textheight);
		
		frameheight = frameheight + textheight + 5;
	end
	frameheight = frameheight + 8;
	WeaponFrame:SetHeight(frameheight);
end